import PIL.Image
from itertools import islice
import sys

ORANGE = (0xF6, 0xB2, 0x1A, 0xFF)
BLUE = (0x2A, 0x31, 0x72, 0xFF)

def rle_encode(entries):
    # rle encode an iterator of booleans into the format the rle decoder gets

    # rle encode the entries
    rle_runs = []
    current = None

    for entry in entries:
        if current is None:
            current = entry
            count = 1

        elif current == entry:
            count += 1

        else:
            rle_runs.append((current, count))
            current = entry
            count = 1

    rle_runs.append((current, count))

    # add a first false entry if not there
    if rle_runs[0][0]:
        rle_runs.insert(0, (False, 0))

    # remove trailing false entries
    if not rle_runs[-1][0]:
        rle_runs.pop()

    # if a run is 16 or larger, split it with 0 length other runs inbetween
    new_runs = []
    for value, length in rle_runs:
        while length > 255:
            new_runs.append((value, 15))
            new_runs.append((not value, 0))
            length -= 255
        new_runs.append((value, length))
    rle_runs = new_runs


    return [length for _, length in rle_runs]

def rle_encode_compressed(entries):
    # rle encode an iterator of values into the format the rle decoder gets

    # rle encode the entries
    rle_runs = []
    current = None

    for entry in entries:
        if current is None:
            current = entry
            count = 1

        elif current == entry:
            count += 1

        else:
            rle_runs.append((current, count))
            current = entry
            count = 1

    rle_runs.append((current, count))

    # add 17 to the last rle run
    color, length = rle_runs.pop()
    length += 17
    rle_runs.append((color, length))

    # remove trailing bg entries
    # if rle_runs[-1][0] == 0:
    #     rle_runs.pop()

    # convert overly long entries
    new_runs = []
    for value, length in rle_runs:
        while length > 63:
            new_runs.append((value, 63))
            length -= 63
        new_runs.append((value, length))

    # encode it
    return [(length | (value << 6)) for value, length in new_runs]

def batched(iterable, n):
    it = iter(iterable)
    while (batch := tuple(islice(it, n))):
        yield batch

def values_to_c(values, name):
    lines = []
    lines.append(f"const unsigned char {name}[{len(values) + 1}] = {{")
    for chunk in batched(values, 8):
        lines.append("    " + ", ".join(f"0x{value:02X}" for value in chunk) + ",")

    lines.append("    0")
    lines.append("};\n")

    return "\n".join(lines)

def values_to_asm(values):
    lines = []
    values = values[:]
    values.append(0)
    for chunk in batched(values, 8):
        lines.append(".byte " + ", ".join(f"0x{value:02X}" for value in chunk))

    return "\n".join(lines)

def main(imagefile):
    image = PIL.Image.open(imagefile)

    width, height = image.size

    data = []

    for y in range(height):
        for x in range(width):
            r,g,b,a = image.getpixel((x, y))

            data.append((r,g,b,a))

    colors = set(data)
    #print(colors)


    yellow = [i == ORANGE for i in data]
    blue = [i == BLUE for i in data]

    color = [1 if i == ORANGE else 2 if i == BLUE else 0 for i in data]


    compressed = rle_encode_compressed(color)

    with open("data.c", "w", encoding="utf-8") as f:
        #f.write(values_to_c(rle_encode(yellow), "rle_yellow_codes"))
        #f.write(values_to_c(rle_encode(blue), "rle_blue_codes"))
        f.write(values_to_c(compressed, "rle_logo_data"))

    with open("data.asm", "w", encoding="utf-8") as f:
        f.write(values_to_asm(compressed))

if __name__ == '__main__':
    main(sys.argv[1])
